analyse_syntaxique(FirstReport, SynTreeWithoutPunctuation):-
	multiword_detector(FirstReport, FirstReport_MW),
	read_sentences(FirstReport_MW, FirstSentence, RestSentences),
	simple_parse(FirstSentence, Parse),
	complex_ng(Parse, ComplexNG),
	complex_ng(ComplexNG, ComplexNG2),
	complex_ng(ComplexNG2, ComplexNG3),
	complex_pg(ComplexNG3, ComplexPG1),
	complex_pg(ComplexPG1, ComplexPG),
        remove_punctuation(ComplexPG, SynTreeWithoutPunctuation).

read_sentences(FirstReport, FirstSentence, RestSentences) :-
	sentences(Sentences, FirstReport, []),
	!,
	member(FirstSentence, Sentences).


% Complex noun groups are linked with variants of de, pour, et and ou.	
complex_ng([], []).
complex_ng([ng(NG1), pg([DE | NG2]) | Rest1], [ng(NG3) | Rest2]) :-
	(DE == de; DE == des; DE == du; DE == d;
	DE == pour; DE == 'Pour'; 
	DE == et; DE == ou),
	append(NG1, [DE | NG2], NG3),
	!,
	complex_ng(Rest1, Rest2).

% Complex noun groups are linked with inf groups.	
complex_ng([ng(NG1), infg([DE | NG2]) | Rest1], [ng(NG3) | Rest2]) :-
	(DE == de; DE == des; DE == du; DE == d),
	append(NG1, [DE | NG2], NG3),
	!,
	complex_ng(Rest1, Rest2).

%Appositive noun groups, excluding pronouns
complex_ng([ng(NG1), ng(NG2) | Rest1], [ng(NG3) | Rest2]) :-
	NG1 = [PRO1 | _],
	NG2 = [PRO2 | _],
	\+ pronoun(PRO1, _, _),
	\+ pronoun(PRO2, _, _), 
	\+ pronoun_rel(PRO1, _, _),
	\+ pronoun_rel(PRO2, _, _), 
	append(NG1, NG2, NG3),
	!,
	complex_ng(Rest1, Rest2).

complex_ng([X | Rest1], [X | Rest2]) :-
	!,
	complex_ng(Rest1, Rest2).

complex_pg([], []).
complex_pg([pg(NG1), pg([DE | NG2]) | Rest1], [pg(NG3) | Rest2]) :-
	(DE == de; DE == des; DE == du; DE == d),
	append(NG1, [DE | NG2], NG3),
	!,
	complex_pg(Rest1, Rest2).
complex_pg([X | Rest1], [X | Rest2]) :-
	!,
	complex_pg(Rest1, Rest2).


simple_parse(Lin, Lout) :-
	group_detector(Lin, Lout),
	!.	

	
multiword_detector(In, Out) :-
	word_stream_1(Beginning, MultiWord, End, In, []),
	append(Beginning, [MultiWord], Head),
	append(Head, Rest, Out),
	multiword_detector(End, Rest),
	!.
multiword_detector(End, End) :- !.

word_stream_1(Beginning, MultiWord, End) -->
	beginning(Beginning),
	multiword(MultiWord),
	end(End).


group_detector(In, Out) :-
	word_stream_2(Beginning, Group, End, In, []),
	group_detector(End, Rest),
	append(Beginning, [Group], Head),
	append(Head, Rest, Out). %, write(Beginning), write(' ' ).
group_detector(End, End).

word_stream_2(Beginning, Group, End) -->
	beginning(Beginning),
	group(Group),
	end(End).

group(infg(Group)) --> inf_group(Group). % {write(Group), nl}.
group(pg(Group)) --> prep_group(Group).
group(vg(Group)) --> infinitive_group(Group). %{write(Group), nl}. % No difference with verb group
group(gg(Group)) --> gerund_group(Group). %, {write(Group), nl}.
group(vg(Group)) --> verb_group(Group).
group(ng(Group)) --> noun_group(Group).
group(rg(Group)) --> adv_group(Group).
group(rp([Group])) --> pronoun_rel(Group).
group(cs([Group])) --> conj_sub(Group).


inf_group([P1, P2 | NG]) --> prep(P1), prep(P2), infinitive_group(NG). %, {write([P1, P2 | NG]), nl}.
inf_group([P | NG]) --> prep(P), infinitive_group(NG). %, {write([P | NG]), nl}.


prep_group([P1, P2 | NG]) --> prep(P1), prep(P2), noun_group(NG). % Sometime prep coumpound ie "par contre" -- Changement
prep_group([P | NG]) --> prep(P), noun_group(NG).

gerund_group([en, CL | VG]) --> [en], clitic(CL), gerund_group(VG).
gerund_group(['En', CL | VG]) --> ['En'], clitic(CL), gerund_group(VG).
gerund_group([en | VG]) --> [en], gerund_group(VG).
gerund_group(['En' | VG]) --> ['En'], gerund_group(VG).
gerund_group(VG) --> gerund_chain(VC), [t], adjective(ADJ), [que],
	{append(VC, [t, ADJ, que], VG)}. %, {write(VG), nl}.
gerund_group(VG) --> gerund_chain(VC), [t], past_participle(PP), [que],
	{append(VC, [t, PP, que], VG)}.
gerund_group(VG) --> gerund_chain(VC), past_participle(PP), [que],
	{append(VC, [PP, que], VG)}.
gerund_group(VG) --> gerund_chain(VC), [t], adjective(ADJ),
	{append(VC, [t, ADJ], VG)}. %, {write(VG), nl}.
gerund_group(VG) --> gerund_chain(VC), [t], past_participle(PP),
	{append(VC, [t, PP], VG)}.
gerund_group(VG) --> gerund_chain(VC), past_participle(PP), {append(VC, [PP], VG)}.
gerund_group([RBN1, CL, V, RBN2]) --> adv_neg(RBN1), clitic(CL), gerund(V), adv_neg(RBN2).
gerund_group([RBN1, V, RBN2]) --> adv_neg(RBN1), gerund(V), adv_neg(RBN2).
gerund_group([CL, V, ADJ]) --> clitic(CL), aux_conj_gerund_be(V), adj(ADJ). %, {write([CL, V, ADJ]), nl}.
gerund_group([CL, V]) --> clitic(CL), gerund(V).
gerund_group([V, ADJ]) --> aux_conj_gerund_be(V), adj(ADJ). %, {write([V, ADJ]), nl}.
gerund_group([V, I]) --> gerund(V),  infinitive(I).
gerund_group(GC) --> gerund_chain(GC).
gerund_group([V]) --> gerund(V).


gerund_chain([RBN, CL, AC, RB1, RB2]) --> adv_neg(RBN), clitic(CL), aux_conj_gerund(AC), adv(RB1), adv(RB2).
gerund_chain([RBN, AC, RB1, RB2]) --> adv_neg(RBN), aux_conj_gerund(AC), adv(RB1), adv(RB2).
gerund_chain([RBN, CL, AC, RB]) --> adv_neg(RBN), clitic(CL), aux_conj_gerund(AC), adv(RB).
gerund_chain([RBN, AC, RB]) --> adv_neg(RBN), aux_conj_gerund(AC), adv(RB).
gerund_chain([RBN, AC]) --> adv_neg(RBN), aux_conj_gerund(AC).		%ne ... que
gerund_chain([CL, AC, RB]) --> clitic(CL), aux_conj_gerund(AC), adv(RB).
gerund_chain([AC, RB]) --> aux_conj_gerund(AC), adv(RB).
gerund_chain([CL, AC]) --> clitic(CL), aux_conj_gerund(AC).
gerund_chain([AC]) --> aux_conj_gerund(AC).


infinitive_group(VG) --> infinitive_chain_2(VC), [t], past_participle(PP), [que],
	{append(VC, [t, PP, que], VG)}.
infinitive_group(VG) --> infinitive_chain_2(VC), past_participle(PP), [que],
	{append(VC, [PP, que], VG)}.
infinitive_group(VG) --> infinitive_chain_2(VC), [t], past_participle(PP),
	{append(VC, [t, PP], VG)}.
infinitive_group(VG) --> infinitive_chain_2(VC), past_participle(PP),
	{append(VC, [PP], VG)}.

infinitive_group(IG) --> infinitive_chain(IC), clitic(C), infinitive(I), past_participle(PP), {append(IC, [C, I, PP], IG)}.
infinitive_group(IG) --> infinitive_chain(IC), clitic(C), infinitive(I),
	{append(IC, [C, I], IG)}.
infinitive_group(IG) --> infinitive_chain(IC), aux_infinitive(I), past_participle(PP),
	{append(IC, [I, PP], IG)}.
infinitive_group(IG) --> infinitive_chain(IC), infinitive(I1), infinitive(I2),
	{append(IC, [I1, I2], IG)}.
infinitive_group(IG) --> infinitive_chain(IC), infinitive(I),
	{append(IC, [I], IG)}.
infinitive_group([CL, I1, I2]) --> clitic(CL), infinitive(I1), infinitive(I2).
infinitive_group([CL, I]) --> clitic(CL), infinitive(I).
infinitive_group([CL, I]) --> clitic(CL), aux_infinitive(I).
infinitive_group([I1, I2]) --> infinitive(I1), infinitive(I2).
infinitive_group([I]) --> infinitive(I).
infinitive_group([I]) --> aux_infinitive(I).

infinitive_chain(VG) --> verb_group(VG).
infinitive_chain([I]) --> infinitive(I).
infinitive_chain([en, train, de]) --> [en, train, de].
infinitive_chain([ne, pas]) --> [ne, pas].


infinitive_chain_2([RBN, CL, AC, RB1, RB2]) -->
	adv_neg(RBN), clitic(CL), aux_infinitive(AC), adv(RB1), adv(RB2).
infinitive_chain_2([RBN, AC, RB1, RB2]) -->
	adv_neg(RBN), aux_infinitive(AC), adv(RB1), adv(RB2).
infinitive_chain_2([RBN, CL, AC, RB]) -->
	adv_neg(RBN), clitic(CL), aux_infinitive(AC), adv(RB).
infinitive_chain_2([RBN, AC, RB]) --> adv_neg(RBN), aux_infinitive(AC), adv(RB).
infinitive_chain_2([RBN, AC]) --> adv_neg(RBN), aux_infinitive(AC).		%ne ... que
infinitive_chain_2([CL, AC, RB]) --> clitic(CL), aux_infinitive(AC), adv(RB).
infinitive_chain_2([AC, RB]) --> aux_infinitive(AC), adv(RB).
infinitive_chain_2([CL, AC]) --> clitic(CL), aux_infinitive(AC).
infinitive_chain_2([AC]) --> aux_infinitive(AC).

verb_chain_be([RBN, CL, AC, RB1, RB2]) --> adv_neg(RBN), clitic(CL), aux_conj_be(AC), adv(RB1), adv(RB2).
verb_chain_be([RBN, AC, RB1, RB2]) --> adv_neg(RBN), aux_conj_be(AC), adv(RB1), adv(RB2).
verb_chain_be([RBN, CL, AC, RB]) --> adv_neg(RBN), clitic(CL), aux_conj_be(AC), adv(RB).
verb_chain_be([RBN, AC, RB]) --> adv_neg(RBN), aux_conj_be(AC), adv(RB).
verb_chain_be([RBN, AC]) --> adv_neg(RBN), aux_conj_be(AC).		%ne ... que
verb_chain_be([CL, AC, RB]) --> clitic(CL), aux_conj_be(AC), adv(RB).
verb_chain_be([AC, RB]) --> aux_conj_be(AC), adv(RB).
verb_chain_be([CL, AC]) --> clitic(CL), aux_conj_be(AC).
verb_chain_be([AC]) --> aux_conj_be(AC).


verb_chain([RBN, CL, AC, RB1, RB2]) --> adv_neg(RBN), clitic(CL), aux_conj(AC), adv(RB1), adv(RB2).
verb_chain([RBN, AC, RB1, RB2]) --> adv_neg(RBN), aux_conj(AC), adv(RB1), adv(RB2).
verb_chain([RBN, CL, AC, RB]) --> adv_neg(RBN), clitic(CL), aux_conj(AC), adv(RB).
verb_chain([RBN, AC, RB]) --> adv_neg(RBN), aux_conj(AC), adv(RB).
verb_chain([RBN, AC]) --> adv_neg(RBN), aux_conj(AC).		%ne ... que
verb_chain([CL, AC, RB]) --> clitic(CL), aux_conj(AC), adv(RB).
verb_chain([AC, RB]) --> aux_conj(AC), adv(RB).
verb_chain([CL, AC]) --> clitic(CL), aux_conj(AC).
verb_chain([AC]) --> aux_conj(AC).


verb_group(VG) --> verb_chain(VC), [t], pp_group(PP), [que],
	{append(VC, [t | PP], VG1), append(VG1, [que], VG)}.
verb_group(VG) --> verb_chain(VC), pp_group(PP), [que],
	{append(VC, PP, VG1), append(VG1, [que], VG)}.
verb_group(VG) --> verb_chain(VC), [t], adj_group(ADJ),
	{append(VC, [t | ADJ], VG)}. %, {write(VG), nl}.
verb_group(VG) --> verb_chain(VC), [t], pp_group(PP),
	{append(VC, [t | PP], VG)}.
verb_group(VG) --> verb_chain(VC), [t], {append(VC, [t], VG)}.
verb_group(VG) --> verb_chain_be(VC), adj_group(ADJ),
	{append(VC, ADJ, VG)}. %, {write(VG), nl}.
verb_group(VG) --> verb_chain(VC), pp_group(PP), {append(VC, PP, VG)}.
verb_group([RBN1, CL, V, RBN2]) -->
	adv_neg(RBN1), clitic(CL), tensed_verb(V), adv_neg(RBN2).
verb_group([RBN1, V, RBN2]) -->
	adv_neg(RBN1), tensed_verb(V), adv_neg(RBN2).
verb_group([RBN, V]) -->
	adv_neg(RBN), tensed_verb(V).
verb_group([CL, V]) --> clitic(CL), tensed_verb(V).
verb_group([V]) --> tensed_verb(V).


% adj_group(-AdjGroup)
% detects a list of words making an adjective
% group and unifies AdjGroup with it

adj_group([RB | AG]) -->
	adv(RB), adj_group(AG).
adj_group([A | AG]) -->
	adj(A), adj_group(AG).
adj_group([A]) --> adj(A).

% pp_group(-PPGroup)
% detects a list of words making a past particple
% group and unifies PPGroup with it

pp_group([RB | PP]) -->
	adv(RB), pp_group(PP).
pp_group([PP]) --> past_participle(PP).

% adv_group(-ADVGroup)
% detects a list of words making an adverb
% group and unifies PPGroup with it

adv_group([RB | ADVG]) -->
	adv(RB), adv_group(ADVG).
adv_group([ADV]) --> adv(ADV).


% noun_group(-NounGroup)
% detects a list of words making a noun group and 
% unifies NounGroup with it
noun_group(NG) -->
	det(D), noun_group_no_det(N), ['-', l],
	{append([D], N, NG1), append(NG1, [l], NG)}.
noun_group(NG) -->
	det(D), noun_group_no_det(N), [l],
	{append([D], N, NG1), append(NG1, [l], NG)}.
noun_group(NG) -->
	det(D), noun_group_no_det(N),
	{append([D], N, NG)}.
noun_group(NG) --> noun_group_no_det(NG).
noun_group([PRO]) --> pronoun(PRO).

noun_group_no_det(NG) --> 
	adj_group(AG1), noun(N), adj_group(AG2), 
	{append(AG1, [N], NG1), append(NG1, AG2, NG)}.
noun_group_no_det(NG) --> 
	adj_group(AG), noun(N),
	{append(AG, [N], NG)}.
noun_group_no_det(NG) --> 
	noun(N), adj_group(AG), 
	{append([N], AG, NG)}.
noun_group_no_det(NG) --> speed(NG).
noun_group_no_det([N]) --> noun(N).
noun_group_no_det(AG) --> adj_group(AG).

beginning(X, Y, Z) :- append(X, Z, Y).
end(X, Y, Z) :- append(X, Z, Y).



tokens(Tokens) --> blank, {!}, tokens(Tokens).
tokens([FirstT | Tokens]) --> token(FirstT), {!}, tokens(Tokens).
tokens([]) --> [].

% A blank is a white space or a control character
blank --> [B], {B =< 32, !}.

% A token is a sequence of alphanumeric characters 
% or another symbol

token(Word) --> alphanumerics(List), {name(Word, List), !}.
token(Symbol) --> other(CSymbol), {name(Symbol, [CSymbol]), !}.

% A sequence of alphanumerics is an alphanumeric character 
% followed by the rest of alphanumerics or a 
% single alphanumeric character.

alphanumerics([L | LS]) --> alphanumeric(L), alphanumerics(LS).
alphanumerics([L]) --> alphanumeric(L).


% Here comes the definition of alphanumeric characters: digits,
% upper-case letters without accent, lower-case letters without 
% accent, and accented characters. Here we only consider
% letters common in French and German

% digits
alphanumeric(D) --> [D], { 48 =< D, D =< 57, !}.

% upper-case letters without accent
alphanumeric(L) --> [L], {65 =< L, L =< 90, !}.

% lower-case letters without accent
alphanumeric(L) --> [L], {97 =< L, L =< 122, !}.

% accented characters
alphanumeric(L) --> [L], {name(A, [L]), accented(A), !}.

accented(L) :-
	member(L, [, , , , , , , , , , , , , '', , , , , , 
	'', '', '', '', '', '', '', '', '', '', '', '', '', '',
	'', '', '', '', '']).
% All other symbols come here
other(39) --> [L], {(L =:= 180; L =:= 146), !}. % Specific processing of 
								% slanted apostrophs replaced by 
								% straight ones
other(Symbol) --> [Symbol], {!}.

sentences([S | RS]) -->
	words_of_a_sentence(S),
	sentences(RS).
sentences([S]) -->	% The last sentence (punctuated)
	words_of_a_sentence(S).
sentences([S]) -->	% Last sentence (no final punctuation)
	words_without_punctuation(S).

words_of_a_sentence([P]) -->
	end_punctuation(P).
words_of_a_sentence([W | RS]) --> 
	word(W),
	words_of_a_sentence(RS).

words_without_punctuation([W | RS]) -->
	word(W),
	words_without_punctuation(RS).
words_without_punctuation([W]) --> [W].

word(W) --> [W].

end_punctuation(P) --> [P], {end_punctuation(P), !}. 

end_punctuation(P) :-
	member(P, ['.', ';', ':', '?', '!']).

read_file(File, List) :-
	see(File),
	read_list(List),
	seen,
	!.

%read_list(-List) reads a list of character until an EOF is encountered
read_list([X|List]) :-
	get0(X),
	X =\= -1,
	!,
	read_list(List).
read_list([]).

%read_line(-Line) reads a line of character until an CR is encountered
read_line([X|Line]) :-
	get0(X),
	X =\= 10,
	!,
	read_line(Line).
read_line([]).


% The classical quicksort(+InputList, -SortedList)

quicksort([], []) :- !.
quicksort([X | LI], LO) :-
	split(X, LI, LSmall, LBig),
	quicksort(LSmall, LSmallSorted),
	quicksort(LBig, LBigSorted),
	append(LSmallSorted, [X | LBigSorted], LO).
	
split(X, [Y | L], [ Y | LSmall], LBig) :-
	before(Y, X),
	!,
	split(X, L, LSmall, LBig).
split(X, [Y | L], LSmall, [Y | LBig]) :-
	!,
	split(X, L, LSmall, LBig).
split(_, [], [], []) :- !.

before(X, Y) :- X @< Y.

count_duplicates(OrderedList, CountedList) :-
	count_duplicates(OrderedList, 1, [], CountedListRev),
	reverse(CountedListRev, CountedList).

count_duplicates([X, X | Ordered], N, Counting, Counted) :-
	N1 is N + 1,
	count_duplicates([X | Ordered], N1, Counting, Counted).
count_duplicates([X | Ordered], N, Counting, Counted) :-
	count_duplicates(Ordered, 1, [[N, X] | Counting], Counted).
count_duplicates([], _, L, L).

number_plate(no_immatriculation) -->
	[Number], [A], [Dept], 
	{number(Number), name(A, L), length(L, N), (N =:= 2; N =:= 3), number(Dept),
	Dept < 100}.

date_value(date_value) -->
	[Day, '/', Month, '/', Year],
	{number(Day), number(Month), number(Year)}.
date_value(date_value) -->
	[Day, '-', Month, '-', Year],
	{number(Day), number(Month), number(Year)}.
date_value(date_value) -->
	[Day, '-', Month, '/', Year],
	{number(Day), number(Month), number(Year)}.

time_value(time_value) -->
	[X],
	{name(X, L), reverse(L, [h | LR]), number(LR)}.
time_value(time_value) -->
	[Hour, h, Minutes],
	{number(Hour), number(Minutes)}.

speed(LS) --> adv(RB1), speed_val(S), adv(RB2), {append([RB1 | S], RB2, LS)}.
speed([RB | S]) --> adv(RB), speed_val(S).
speed(LS) --> speed_val(S), adv(RB), {append(S, RB, LS)}.
speed(S) --> speed_val(S).

speed_val([Number, kmh]) -->
	[Number], [km], ['/'], [h], 
	{number(Number)}.
speed_val([Number, kmh]) -->
	[Number], ['Km'], ['/'], [h], 
	{number(Number)}.

distance(distance_value) -->
	[Number], [mtres], 
	{number(Number)}.

 

% drule(Head, Dependent, Function)

drule(gg, pg, _).
drule(gg, ng, _).
drule(gg, rg, _).
drule(gg, infg, _).

drule(pg, pg, _).

drule(vg, ng, _).
drule(vg, pg, _).
drule(vg, infg, _).
drule(vg, gg, _).
drule(vg, rg, _).

drule(infg, ng, _).
drule(infg, rg, _).

drule(ng, rg, _).
drule(ng, pg, _).

write_list([X | R]) :-
	write(X), nl,
	write_list(R).
write_list([]).

print_variables([X | Result]) :-
	X =.. [w | [L1, L2, _]],
	(var(L2),
	L1 \== '.',
	L1 \== ',',
	write(X), nl
	;
	true),
	print_variables(Result), !.
print_variables([]).



parse(InputList, Result) :-
	parse(InputList, [], [], ReversedResult),
	reverse(ReversedResult, Result),
	!.

% parse(+InputList, +HeadList, +WordList, -Result)

% We search dependents of W in HeadList
% and a head for W in WordList
parse([W | InputList], HeadList, WordList, Result) :-
	W =.. [CAT | L],
	% We search the headlist for dependents
	% head-dependent order is only right to left
	% for subordinating conjunctions
	search_headlist_for_dependents(W, HeadList, NewHeadList, WordList),
	(
	% We now search the wordlist backward to find a head
	% head-dependent order is only left to right for pronouns
	% should fail for pronoun
	head_dependent_right_to_left_order(L),
	search_wordlist_for_a_head(W, WordList, Head,  Function),
	parse(InputList, NewHeadList, [w(W, Head, Function) | WordList], Result)
	;
	parse(InputList, [W | NewHeadList], [w(W, Head, Function) | WordList], Result)
	).
parse([], _, WordList, WordList).

search_headlist_for_dependents(W, [W1 | HeadList], NewHeadList, WordList) :-
	W =.. [CAT | L],
	W1 =.. [CAT1 | L1],
	head_dependent_left_to_right_order(L),
	drule(CAT, CAT1, F),
	% We instantiate the relation in WordList
	member(w(W1, W, F), WordList),
	search_headlist_for_dependents(W, HeadList, NewHeadList, WordList).
search_headlist_for_dependents(_, HL, HL, WordList).


% search_wordlist_for_a_head(+Word, +WordList, -Head, -Function),
% Tries to find a head in WordList

% We look for the first word that does not depend on the
% current word

% Changement a partir d'ici

search_wordlist_for_a_head(W, [w(H, _, _)  | WordList] , H, F) :-
	H =.. [POS_H|_],
	W =.. [POS | _],	
	drule(POS_H, POS, F).
	
search_wordlist_for_a_head(W, [w(H1, H2, _)  | WordList] , H, F) :-
	nonvar(H2), 	%H2 must be instantiated
	H1 \== H2,
	search_wordlist_for_a_head(W, H2, [w(H1, H2, _)  | WordList] , [H2, H1], H, F).

search_wordlist_for_a_head(W, H, WordList, HeadList, H, F) :- % We must maintain HeadList to avoid loops
	nonvar(H),	
	H =.. [POS_H|_],
	W =.. [POS | _],
	drule(POS_H, POS, F).
search_wordlist_for_a_head(W, H1, WordList, HeadList, H, F) :-
	member(w(H1, H2, _), WordList),
	nonvar(H2),
	\+ member(H2, HeadList),
	search_wordlist_for_a_head(W, H2, WordList, [H2 | HeadList], H, F).
% Fin changement ici %%

head_dependent_right_to_left_order(L) :-
	L = [[X]],
	!,
	\+ pronoun(X, _, _),
	\+ clitic(X, _, _),
	\+ pronoun_rel(X, _, _).
head_dependent_right_to_left_order(L).


head_dependent_left_to_right_order(L):-
	L = [[X]],
	!,
	\+ conj_sub(X, _, _).
head_dependent_left_to_right_order(L).


% Clause parsing
% clause_parse(+InputList, -Result)

clause_parse(InputList, InputList3) :-
% We first examine the sentence from left to right to find dependents 
% to rel pronouns and sub conjunctions (both considered as 
% prepositions in front of a clause
	search_head_to_dependents(InputList, drule(cs, vg, _)),
	search_head_to_dependents(InputList, drule(rp, vg, _)),
% and to find a head to gg
	search_dependent_to_head(InputList, drule(vg, gg, _)),
% Then we reverse the sentence and search heads to cs and rp
	reverse(InputList, InputList2),
	search_dependent_to_head(InputList2, drule(vg, rp, _)),
	search_dependent_to_head(InputList2, drule(vg, cs, _)),
	reverse(InputList2, InputList3),
% Last we search a head to fronted subordinate clauses
	search_dependent_to_head(InputList3, drule(vg, cs, _)),
	search_head_to_dependents(InputList3, drule(vg, gg, _)).

	
% We search dependents of W in HeadList
% and a head for W in WordList
search_head_to_dependents([W | InputList], Rule) :-
	dependents(W, Rule, InputList),
	search_head_to_dependents(InputList, Rule), !.
search_head_to_dependents([], _).

search_dependent_to_head([W | InputList], Rule) :-
	head(W, Rule, InputList),
	search_dependent_to_head(InputList, Rule), !.
search_dependent_to_head([], _).


% We search a dependent for W
dependents(W, drule(CAT, CAT1, _), [W1  | InputList]) :-
	W =.. [w | [L | _]],
	L =.. [CAT | _],
	W1 =.. [w | [L1, L2, _]],
	var(L2),
	L1 =.. [CAT1 | _],
	% We instantiate the relation
	L2 = L.
dependents(W, Rule, [W1  | InputList]) :-
	dependents(W, Rule, InputList).
dependents(W, Rule, []).

% We search a head for W
head(W, drule(CAT1, CAT, _), [W1  | InputList]) :-
	W =.. [w | [L, L2, _]],
	var(L2),
	L =.. [CAT | _],
	W1 =.. [w | [L1, L3, _]],
	L1 =.. [CAT1 | _],
	L3 \== L, % Quick check. In fact, there should not be a link between the two words
	% We instantiate the relation
	L2 = L1.
head(W, Rule, [W1  | InputList]) :-
	head(W, Rule, InputList).
head(W, _, []).

crule(cs, vg, _).

grule(vg, gg, _).


% Enlve la ponctuation d'une liste
remove_punctuation([], []).
remove_punctuation([X | L1], L2) :-
	end_punctuation(X),
	!,
	remove_punctuation(L1, L2).
remove_punctuation([X | L1], [X | L2]) :-
	remove_punctuation(L1, L2).

