% Le fichier references.pl
% dtermine les rfrences aux objets

% La rsolution de rfrences recherche l'objet dans la base de donnes.
% On traite simplement la compatibilit smantique.
% On ne s'occupe pas de la position de l'avatar.  faire dans l'avenir
% resolution_reference(+PositionInitiale, +FormeSynt, -Objet)

resolution_reference(_, [vg([_]), pg([_ | GNominal])] , Objet) :-
    compatibilite_semantique(GNominal, GNominal, Objet),
    !.
resolution_reference(_, [vg([_]), ng(GNominal)] , Objet) :-
    compatibilite_semantique(GNominal, GNominal, Objet),
    !.
resolution_reference(_, _, []).
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Recherche de l'objet gnrique vers lequel on se dirige
% compatibilite_semantique(+LTypeObjet, +PhraseEntree, -Objet).
% Considre chacun des mots de la phrase comme un objet possible (1er param.)
% et vrifie que ses caractristiques sont compatibles avec la liste de mots 

compatibilite_semantique([TypeObjet | _], ListeMots, Objet):-
	write(TypeObjet), nl,
	liste_objets(TypeObjet, LObjets),
	LObjets \== [], !,
	compatibilite_adjectifs(TypeObjet, LObjets, ListeMots, Objet).
compatibilite_semantique([_ | L], ListeMots, Objet):-
	compatibilite_semantique(L, ListeMots, Objet),
	!.
compatibilite_semantique([], [], _).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Pour choisir le bon objet
% Actuellement, on ne tient compte que des adjectifs.
% Plus tard choisir en fonction de la distance.
% compatibilite_adjectifs(+TypeObjet, +LObjets, +ListeMots, -PredObjet).
% test de la compatibilit smantique par contraintes descendantes.
% D'abord, l'adjectif est dans la liste de mots ou bien
% il n'y est pas.

compatibilite_adjectifs(TypeObjet, LObjets, ListeMots, PredObjet) :-
	member(Objet, LObjets), % On choisit un objet
	PredObjet =.. [objet, [TypeObjet, Objet, _, _, _, _, _, _, _, LAdject]],
	monde(X),
	member(PredObjet, X), % On retrouve ses adjectifs dans le monde
	write(LAdject),
	write(ListeMots),nl,
	intersection(LAdject, ListeMots, Intersection), % Sont-ils dans la liste de mots?
	Intersection \== [].
compatibilite_adjectifs(TypeObjet, [Objet | _], _, PredObjet) :-
	PredObjet =.. [objet, [TypeObjet, Objet | _]],
	monde(X),
	member(PredObjet, X).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% destination(+Prep, +Objet, +PositionInitiale, -PositionFinale, -OrientationFinale)
% doit corriger les coordonnes de l'objet
% ou l'on va en fonction des prepositions
% Devant != derrire, ....
% Objet est donn par objet([Type, Nom, X0, Y0, Z0, VX0, VY0, VZ0, G, H])

destination(, objet([_, _, X0, Y0, Z0 | _]), _, [X1, Y1, Z1], [0, 0, 0]) :-
	X1 is X0,
	Y1 is Y0,
	Z1 is Z0. % On considre qu'on va jusqu'au pied de l'objet

destination(au, objet([_, _, X0, Y0, Z0 | _]), _, [X1, Y1, Z1], [0, 0, 0]) :-
	X1 is X0,
	Y1 is Y0,
	Z1 is Z0. % On considre qu'on va jusqu'au pied de l'objet

destination(vers, objet([_, _, X0, Y0, Z0 , _, _, _, G, _]), [X, Y, Z], [X1, Y1, Z1], [VX1, VY1, VZ1]) :-
	% La direction de l'avatar est celle du mouvement.
	VX1 is X0 - X, VY1 is Y0 - Y, VZ1 is Z0 - Z,
	distance([VX1, VY1, VZ1], Distance),
	Distance > G,
	% On avance en direction de l'objet
	%  une distance qui tient compte de la taille de l'objet
	Proximite is abs(Distance - G)/Distance,
	X1 is Proximite * (X0 - X) + X,
	Y1 is Y0,	% On considre que l'on est sur le plan
	Z1 is Proximite * (Z0 - Z) + Z,
	!.

destination(vers, objet([_, _, X0, Y0, Z0, _, _, _, G, _]), [X, Y, Z], [X1, Y1, Z1], [VX1, VY1, VZ1]) :-
	VX1 is X0 - X, VY1 is Y0 - Y, VZ1 is Z0 - Z,
	distance([VX1, VY1, VZ1], Distance),
	Distance =< G,
	% On reste sur place si on est suffisamment prs
	X1 is X, Y1 is Y, Z1 is Z,
	!.


destination(devant, objet([_, _, X0, Y0, Z0, VX0, VY0, VZ0, G, _]), _, [X1, Y1, Z1], [VX1, VY1, VZ1]) :-
	% direction de l'avatar  la fin du mouvement.
	% Elle est oppose  celle de l'objet
	VX1 is -VX0, VY1 is -VY0, VZ1 is -VZ0,
	distance([VX0, VY0, VZ0], N), 
	X1 is X0 + (VX0*G)/N,
	Y1 is Y0,	% On considre que l'on est sur le plan
	Z1 is Z0 + (VZ0*G)/N. 

destination(derrire, objet([_, _, X0, Y0, Z0, VX0, VY0, VZ0, G, _]), _, [X1, Y1, Z1], [VX1, VY1, VZ1]) :-
	% direction de l'avatar  la fin du mouvement.
	% Elle est dans l'axe de l'objet
	VX1 is VX0, VY1 is VY0, VZ1 is VZ0,
	distance([VX0, VY0, VZ0], N), 
	X1 is X0 - (VX0*G)/N,
	Y1 is Y0,	% On considre que l'on est sur le plan
	Z1 is Z0 - (VZ0*G)/N. 

destination(par_dessus, objet([_, _, X0, Y0, Z0, _, _, _, G, _]), [X, Y, Z], [X1, Y1, Z1], [VX1, VY1, VZ1]) :-
	% La direction de l'avatar est celle du mouvement.
	VX1 is X0 - X, VY1 is Y0 - Y, VZ1 is Z0 - Z,
	distance([VX1, VY1, VZ1], N), 
	X1 is X + (VX1*G)/N,
	Y1 is Y0,	% On considre que l'on est sur le plan
	Z1 is Z + (VZ1*G)/N, % En fait X et Z ne sont pas utiliss. Le calcul est fait par l'intermdiaire de la parabole.
	!.

destination(_droite_de, objet([_, _, X0, Y0, Z0, VX0, VY0, VZ0, G, _]), _, [X1, Y1, Z1], [VX1, VY0, VZ1]) :-
	VX1 is -VZ0, VZ1 is VX0,
	distance([VX0, VY0, VZ0], N), 
	X1 is X0 + (VZ0*G)/N,
	Y1 is Y0,	% On considre que l'on est sur le plan
	Z1 is Z0 - (VX0*G)/N.

destination(_gauche_de, objet([_, _, X0, Y0, Z0, VX0, VY0, VZ0, G, _]), _, [X1, Y1, Z1], [VX1, VY0, VZ1]) :-
	VX1 is VZ0, VZ1 is -VX0,
	distance([VX0, VY0, VZ0], N), 
	X1 is X0 - (VZ0*G)/N,
	Y1 is Y0,	% On considre que l'on est sur le plan
	Z1 is Z0 + (VX0*G)/N.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vecteur directeur de la droite nous reliant  notre but
% Ceci nous renvoie aussi l'angle que l'on fait avec notre point originel
% (Lancement du programme)

vecteur_directeur([X0, Y0, Z0], [X1, Y1, Z1], [VX, VY, VZ]) :-
	VX is X1 - X0,
	VY is Y1 - Y0,
	VZ is Z1 - Z0.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Distance de l'objet a atteindre

distance([X, Y, Z], R):-
	R is sqrt((X*X) + (Y*Y) + (Z*Z)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% fonction calculant le point d'arrive en fonction du point de dpart dans le plan X Z
% --------------------------------------------------------------------------
% avance_position(+PositionInitiale, +Distance, +CoordonnesFinales).
% La distance est de 7 normalement

avance_position([X0, Y0, Z0, Angle0], Distance, [X1, Y0, Z1]) :-
	X1 is X0 - Distance*sin(Angle0),
	Z1 is Z0 - Distance*cos(Angle0).


% Calcul des angles pour les rotations
% angles(+AngleInitial, +Vecteur, -AngleVecteur)

angles(AngleInitial, [X1, _, Z1], Rep):-
	[X0, _, Z0] = [0, 0, -1],	% Direction du point de vue quand l'angle est  0
	SIN is -(Z1*X0) + (X1*Z0),
	COS is (X1*X0) + (Z1*Z0),
	N1 is sqrt((X1*X1) + (Z1*Z1)),
	N2 is sqrt((X0*X0) + (Z0*Z0)),
	write(SIN), nl,
	write(COS), nl,
	angle2(SIN, COS, N1, N2, AngleInitial, Rep).

% angle2(SIN, COS, N1, N2, AngleInitial, Rep)
angle2(_, _, 0, _, _, 0) :- !.
angle2(_, _, _1, 0, _, 0) :- !.
angle2(SIN, 0, _, _, _, Rep) :- SIN >= 0, Rep is pi/2, !.
angle2(SIN, 0, _, _, _, Rep) :- SIN < 0, Rep is -pi/2, !.
angle2(SIN, COS, _, _, _, Rep):- COS > 0, Rep is atan(SIN/COS), !.
angle2(SIN, COS, _, _, _, Rep):- COS < 0, Rep is atan(SIN/COS) + pi, !.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% De quel cote on tourne ?
% ou(+Direction, +PositionInitiale, +AngleRotation, -AngleFinal)

ou(_droite, [_, _, _, Angle0], AngleRotation, Angle1):-
	Angle1 is Angle0 - AngleRotation.
ou(_gauche, [_, _, _, Angle0], AngleRotation, Angle1):-
	Angle1 is Angle0 + AngleRotation.


% Apprentissage d'un nouvel objet. On met  jour la base de donnes et le fichier
% monde.pl

ajoute_objet(NObjet) :-
	monde(Monde),
	append(Monde, [NObjet], NMonde),
	retract(monde(Monde)),
	asserta(monde(NMonde)),
	ulysse_folder(DossierUlysse),
	monde_prolog(FichierMonde),
	concatenate(DossierUlysse, FichierMonde, CMONDE),
	tell(CMONDE),
	write(monde(NMonde)), write('.'), nl,
	told,
	!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ne sert pas encore
% Ceci utilisera strips quand il sera implant !

/*
:-[strips].
plannifieur([X|Ordre],[Y|Reponse]) :- plannifieur(Ordre,Reponse).
plannifieur([],Reponse) :- Reponse=[].
*/
