% Fichier d'utilitaires divers

% liste_objets(+Type, -LObjets) retourne le liste des objets d'un certain type, par exemple
% la liste des boules.
liste_objets(Type, LNoms) :-
	monde(LObjets),
	!,
	findall(Nom, member(objet([Type, Nom | _]), LObjets), LNoms).


% Concatne deux atomes
concatenate(Folder, File, Path) :-
	string_concat(Folder, File, PathList),
	string_to_atom(PathList, Path),
	!.

% calcul de a et b pour l'quation d'une droite y = ax + b
%------------------------------------------------------------
% calcul_a_b(+[X0, Y0, Z0], +[X1, Y1, Z1], -A, -B). La droite est dans le plan
% On ne tient compte que de x et z

calcul_a_b([0, _, Z0], [0, _, _], 0, Z0) :- !.
calcul_a_b([X0, _, Z0], [X1, _, Z1], A, B) :-
    !,
    A is (Z1 - Z0)/(X1 - X0),
    B is Z0 - X0*A.



% calcul du discriminant pour l'intersection entre une quation de cercle et une droite
%-----------------------------------------------------------------------------------------
% discriminant(+Coordonnes, +TailleRayon, +CoeffADroite, +CoeffBDroite, -Delta) 
% Si ngatif pas d'intersection, si 0, une solution, si positif, deux points d'intersection
discriminant([X, _, Z], TailleRayon, A, B, Delta):-
        T1 is TailleRayon/2,
        D1 is T1*T1 * (1+A*A),
        D2 is X*A + (B - Z),
        D3 is D2*D2 ,
        Delta is D1 - D3.

% calcul des points d'intersections de la trajectoire projete dans le plan X, Z 
% et du cercle dfini par la taille de l'objet (le discriminant). La trajectoirerevient donc  une droite dans le plan X Z
% points_intersection(+PositionObjet, +A, +B, +Delta, -PositionIntersection1, -PositionIntersection2)
%----------------------------------


points_intersection([X, _, Z], A, B, Delta, [X1, Z1], [X2, Z2]) :-
        D1 is A*(B - Z) - X,
        D2 is 1 + A*A,
        D3 is sqrt(Delta),
        X1 is (-D1 + D3)/D2,
        X2 is (-D1 - D3)/D2,
        Z1 is A*X1 + B,
        Z2 is A*X2 + B.
%intersection avec segment 
%-------------------------
% intersection(+Point, +DbutSegment, +FinSegment, -Reponse)
% Le point est sur la droite dfini par le segment
% Point est donn par ses coordonnns sur le plan X Z

intersection([X, _], [X1, _, _], [X2, _, _], 1) :-
    X1 < X2,  X1 < X,  X < X2.
intersection([X, _], [X, _, _], [X2, _, _], 1) :-
    X < X2.
intersection([X, _], [X1, _, _],[X, _, _], 1) :-
    X1 < X.
intersection(_, [X1, _, _], [X2, _, _], 0) :-
    X1 < X2.
intersection([X, _], [X1, _, _], [X2, _, _], 1):-
    X1 > X2, X2 < X, X < X1.
intersection([X, _], [X, _, _], [X2, _, _], 1) :-
    X > X2.
intersection([X, _], [X1, _, _], [X, _, _], 1):-
    X1 > X.
intersection(_, [X1, _, _], [X2, _, _], 0) :-
    X1 > X2.
intersection([X, Z], [X, _, Z1], [X, _, Z2], Rep) :-
    intersection_aux(Z, Z1, Z2, Rep).
intersection(_, [X, _, _], [X, _, _], 0).

% Si la droite est verticale
intersection_aux(Z, Z1, Z2, 0):- Z1 < Z2, Z < Z1. 
intersection_aux(Z, Z1, Z2, 0):- Z1 < Z2, Z > Z2. 
intersection_aux(Z, Z, Z2, 1):- Z < Z2. 
intersection_aux(Z, Z1, Z, 1):- Z1 < Z. 
intersection_aux(Z, Z1, Z2, 1):- Z1 < Z2, Z1 < Z, Z < Z2. 
intersection_aux(Z, Z1, Z2, 0):- Z1 > Z2, Z > Z1. 
intersection_aux(Z, Z1, Z2, 0):- Z1 > Z2, Z < Z2. 
intersection_aux(Z, Z, Z2, 1):- Z > Z2. 
intersection_aux(Z, Z1, Z, 1):- Z1 > Z. 
intersection_aux(Z, Z1, Z2, 1):- Z1 > Z2, Z1 > Z, Z > Z2. 


% traite les rponses obtenues par intersection
%-------------------------------------------------
traite_rep(1, 1, 1).
traite_rep(1, 0, 1).
traite_rep(0, 1, 1).
traite_rep(0, 0, 0).


/*
intersection([], A, []) :- !.
intersection([A|B], C, D) :-
	member(A, C), !,
	D=[A|E],
	intersection(B, C, E).
intersection([A|B], C, D) :-
	intersection(B, C, D).
*/

% Le  fichier monde.pl est effac  chaque addition d'un nouvel objet
% son contenu est compos d'objets dont la structure est 
% objet([Type, Nom, X, Y, Z, VX, VY, VZ, Taille, [LAdjectifs]]).
% X, Y, Z sont les coordonnes et VX, VY, VZ est le vecteur d'orientation de l'objet
